/*
 * CrudContainer
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.filter.core;

import dk.apaq.filter.FiltrationItem;

/**
 *
 */
public class BetweenFilter<T> extends AbstractFilterComponent {

    private final T startValue;
    private final T endValue;

    public BetweenFilter(String propertyId, T startValue, T endValue) {
        super(propertyId);
        this.startValue = startValue;
        this.endValue = endValue;
    }

    public T getStartValue() {
        return startValue;
    }

    public T getEndValue() {
        return endValue;
    }

    @Override
    public boolean passesFilter(FiltrationItem item) {
        Object value = item.getFiltrationProperty(this.getPropertyId());
        int comparedToStart = compareValues(startValue, value);
        int comparedToEnd = compareValues(endValue, value);

        return comparedToStart >= 0 && comparedToEnd <= 0;
    }

    @SuppressWarnings({"unchecked", "rawtypes"})
    protected int compareValues(Object value1, Object value2) {
        if (null == value1) {
            return null == value2 ? 0 : -1;
        } else if (null == value2) {
            return 1;
        } else if (value1 instanceof Comparable
                && value2.getClass().isAssignableFrom(value1.getClass())) {
            return -( (Comparable) value1 ).compareTo(value2);
        }
        throw new IllegalArgumentException("Could not compare the arguments: "
                + value1 + ", " + value2);
    }
}
