/*
 * CrudContainer
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.filter.sort;

import dk.apaq.filter.FiltrationItem;

/**
 * A entry used in a Sorter.
 */
public class SorterEntry {

    private final String propertyId;
    private final SortDirection direction;

    /**
     * Creates a new SorterEntry.
     * @param propertyId The id for sorting.
     * @param direction The direction of sorting.
     */
    public SorterEntry(String propertyId, SortDirection direction) {
        if(propertyId == null) {
            throw new IllegalArgumentException("propertyid cannot be null.");
        }

        if(direction == null) {
            throw new IllegalArgumentException("direction cannot be null.");
        }

        this.propertyId = propertyId;
        this.direction = direction;
    }

    /**
     * Retrieves the direction.
     * @return The direction
     */
    public SortDirection getDirection() {
        return direction;
    }

    /**
     * Retrieves the propertyid to use for sorting.
     * @return The propertyid.
     */
    public String getPropertyId() {
        return propertyId;
    }

    /**
     * Compares 2 items.
     * @param item1 First item.
     * @param item2 Second item.
     * @return The result for comparing the 2 items.
     */
    public SortResult compare(FiltrationItem item1, FiltrationItem item2) {
        Object prop1 = item1.getFiltrationProperty(propertyId);
        Object prop2 = item2.getFiltrationProperty(propertyId);

        if(prop1 instanceof Comparable) {
            int result = ((Comparable)prop1).compareTo(prop2);
            if (result==0) {
                return SortResult.Equal;
            }

            if(direction == SortDirection.Ascending) {
                return result<0 ? SortResult.LessThan : SortResult.GreaterThan;
            } else {
                return result>0 ? SortResult.LessThan : SortResult.GreaterThan;
            }
        }
        return SortResult.Equal;
    }


}
